<?php

/**
 * DocBlock for OcApi class
 *
 * Wrapper class to create an open API towards Open Content
 *
 * @author Infomaker Scandinavie AB
 *
 */

require_once( 'oc_connection.php' );
require_once dirname( __FILE__ ) . '/../oc_objects/oc_ad.php';
require_once dirname( __FILE__ ) . '/../oc_objects/oc_article.php';
require_once dirname( __FILE__ ) . '/../oc_objects/oc_image.php';
require_once dirname( __FILE__ ) . '/../oc_objects/oc_article_custom_post_factory.php';
require_once dirname( __FILE__ ) . '/../oc_objects/oc_objects_collection.php';

/*
|--------------------------------------------------------------------------
| Open Content API Class
| Used to interact with Open Content
|--------------------------------------------------------------------------
*/

class OcAPI extends OcConnection {

	private static $_oc = null;

	/**
	 * Public construct
	 * Will make a singleton Open Content object
	 *
	 * @scope public
	 */
	public function __construct() {
//		OcAPI::$_oc = OcAPI::$_oc === null ? new OpenContent() : OcAPI::$_oc;
		OcAPI::$_oc = OcAPI::$_oc === null ? OpenContent::getInstance() : OcAPI::$_oc;
		parent::__construct( OcAPI::$_oc );
	}

	/**
	 * Function to prepare a query
	 * Will append a prepared query onto the OC URL
	 *
	 * @scope public
	 *
	 * @param        $query
	 * @param string $start
	 * @param string $limit
	 * @param string $sort
	 *
	 * @return $this
	 */
	public function prepare_query( $query, $start = "", $limit = "", $sort = "" ) {
		$this->oc_url = $this->prep_oc_query( $query, $start, $limit, $sort );
		return $this;
	}

	/**
	 * Function to prepare a Widgets oc query
	 *
	 * @scope public
	 *
	 * @param $instance
	 *
	 * @return $this
	 */
	public function prepare_widget_query( $instance ) {
		$query = isset( $instance['oc_query'] ) ? $instance['oc_query'] : null;
		$start = isset( $instance['oc_query_start'] ) ? $instance['oc_query_start'] : null;
		$limit = isset( $instance['oc_query_limit'] ) ? $instance['oc_query_limit'] : null;
		$sort  = isset( $instance['oc_query_sort'] ) ? $instance['oc_query_sort'] : null;

		$this->oc_url = $this->prep_oc_query( $query, $start, $limit, $sort );
		return $this;
	}

	/**
	 * Function to prepare a suggest query
	 *
	 * @scope public
	 *
	 * @param $instance
	 */
	public function prepare_suggest_query( $instance ) {
		$suggest = isset( $instance['oc_suggest_field'] ) ? $instance['oc_suggest_field'] : null;
		$query   = isset( $instance['oc_query'] ) ? $instance['oc_query'] : null;

		$this->prepare_oc_suggest_query( $suggest, $query );
	}

	/**
	 * Function used to prepare a query before its sent to Event Notifier
	 *
	 * @scope public
	 *
	 * @param $instance
	 *
	 * @return string
	 */
	public function prepare_notifier_query( $instance ) {
		$query = isset( $instance['oc_query'] ) ? $instance['oc_query'] : null;
		$start = isset( $instance['oc_query_start'] ) ? $instance['oc_query_start'] : null;
		$limit = isset( $instance['oc_query_limit'] ) ? $instance['oc_query_limit'] : null;
		$sort  = isset( $instance['oc_query_sort'] ) ? $instance['oc_query_sort'] : null;
		return $this->_build_query_string( $query, $start, $limit, $sort, $url_encoded = true );
	}

	/**
	 * Function to get OC Content types
	 *
	 * @scope public
	 *
	 * @return array
	 */
	public function get_content_types() {
		return $this->ajax_get_content_types();
	}

	/**
	 * Function to get a collection with Raw data from Open Content
	 *
	 * @scope public
	 *
	 * @param        $query
	 * @param string $start
	 * @param string $limit
	 * @param string $sort
	 *
	 * @return OcObjectsCollection
	 */
	public function get_oc_objects_collection( $query, $start = "", $limit = "", $sort = "" ) {

		$this->oc_url = $this->prep_oc_query( $query, $start, $limit, $sort );

		$result = $this->get_json_result();

		$collection = new OcObjectsCollection();
		$collection->set_json( $result );

		if ( $result !== null && $result !== '401' ) {

			$json_result = json_decode( $result );
			$hits        = $json_result->hits;

			foreach ( $hits->hits as $object ) {
				switch ( strtolower( $object->versions[0]->properties->contenttype[0] ) ) {
					case 'ad':
						$collection->ad_object_to_collection( $this->create_ad_object( $object ) );
						break;
					case 'article':
						$article = $this->create_article_object( $object );
						new OcArticleCustomPostFactory( $article );
						$collection->ad_object_to_collection( $article );
						break;
					case 'image':
						$collection->ad_object_to_collection( $this->create_image_object( $object ) );
						break;
				}
			}
		}

		return $collection;
	}

	/**
	 * Function to make a raw unfiltered query against Open Content
	 * @param  String $query [description]
	 * @param  string $start [description]
	 * @param  string $limit [description]
	 * @param  string $sort  [description]
	 * @return [JSON]        [description]
	 */
	function get_raw_query( $query, $start = "", $limit = "", $sort = "" ){
		$this->oc_url = $this->prep_oc_query( $query, $start, $limit, $sort );

		return $this->get_json_result();
	}

	/**
	 * Function to get full OC Query url data
	 *
	 * @scope public
	 *
	 * @param $url
	 *
	 * @return null|string
	 */
	public function get_url_content( $oc_query_url ) {
		return $this->_get_remote_data( $oc_query_url );
	}

	/**
	 * Function to tell if we are using Notifier or not
	 *
	 * @scope public
	 *
	 * @return mixed
	 */
	public function notifier_active() {
		return self::$_oc->getUseOcNotifier();
	}

	
}
