<?php

/**
 * Class OcMetaWriter
 */
class OcMetaWriter {

	// Class properties.
	private $metadata_filename  = 'everyware-wp-meta-0.1.xml';
	private $metadata_mimetype  = 'everyware/wp-meta';
	private $edit_user_exists   = false;

	function __construct() {

		if( defined( 'OC_EDIT_USER' ) && defined( 'OC_EDIT_PASSWORD' ) ) {
			$this->edit_user_exists = true;
		}
	}

	/**
	 * Sends the XML to Open Content for indexing.
	 * @param $post
	 * @return bool
	 */
	public function update_metadata_xml( $post ) {

		// Check that user with edit priveliges exists otherwise do nothing.
		if( $this->edit_user_exists ) {

			$uuid   = OcUtilities::get_uuid_by_post_id( $post->ID );
			$xml    = $this->generate_metadata_xml_for_post( $post, $uuid );
			$result = $this->put_metadata_xml( $uuid, $xml );

			if( isset( $result ) ) {
				return true;
			}
			else {
				return false;
			}
		}
		else {

			return false;
		}
	}

	public function update_metadata_xml_by_uuid( $uuid, $write_data ) {
		if( $this->edit_user_exists ) {
			$xml 		= $this->generate_metadata_xml_for_uuid( $uuid, $write_data );
			$result = $this->put_metadata_xml( $uuid, $xml );

			if( isset( $result ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Looks at the given WP Article and generates an XML for it.
	 * @param $post
	 * @param $uuid
	 * @return string
	 */
	private function generate_metadata_xml_for_post( $post, $uuid ) {

		$site_shortname = OcUtilities::get_site_shortname();
		$post_status    = $post->post_status;
		$link           = get_permalink( $post->ID );
		$current_xml    = $this->get_current_metadata_xml( $uuid );
		$trashed        = $post_status === 'trash' ? $site_shortname : 'false';

		if( isset( $current_xml ) ) { // If metadata file already exists in OC.

			$xml = simplexml_load_string( $current_xml );
			foreach( $xml->site as $site ) {

				// Check if current site already exists in the XML.
				if( isset( $site['shortname'] ) && (string)$site['shortname'] === $site_shortname ) {

					// Delete it if it exists.
					$dom = dom_import_simplexml( $site );
					$dom->parentNode->removeChild( $dom );
					break;
				}
			}
		}
		else { // If metadata does not exist, create a new one.
			$xml    = new SimpleXMLElement( '<sites/>' );
		}

		$site = $xml->addChild( 'site' );
		$site->addAttribute( 'shortname', $site_shortname );
		$site->addChild( 'trashed', $trashed );
		$site->addChild( 'url', $link );
		return $xml->asXML();
	}

	/**
	 * Takes write data from parameter and adds data to uuid XML in OC.
	 * @param $uuid
	 * @param $write_data
	 * @return string
	 */
	private function generate_metadata_xml_for_uuid( $uuid, $write_data ) {

		$site_shortname = '';
		$link           = '';
		$current_xml    = $this->get_current_metadata_xml( $uuid );
		$trashed        = 'false';

		if( isset( $current_xml ) ) { // If metadata file already exists in OC.
			$xml = simplexml_load_string( $current_xml );
		}
		else { // If metadata does not exist, create a new one.
			$xml    = new SimpleXMLElement( '<sites/>' );
		}

		if( isset( $write_data['links'] ) ) {

			if( $xml->urls[0] !== null ) {
				$links = $xml->urls;
				foreach( $links->url as $link ) {

					$xml_link = (string)$link;
					if( in_array( $xml_link, $write_data['links'] ) ) {
						$dom = dom_import_simplexml( $link );
						$dom->parentNode->removeChild( $dom );
					}
				}
			}
			else {
				$links = $xml->addChild( 'urls' );
			}

			foreach( $write_data['links'] as $link ) {
				$links->addChild( 'url', $link );
			}
		}
		
		return $xml->asXML();
	}

	/**
	 * Get the current metadata file for the given object.
	 * @param $uuid
	 * @return string
	 */
	private function get_current_metadata_xml( $uuid ) {

		$path = 'objects/' . $uuid . '/files/' . $this->metadata_filename;
		return $this->perform_oc_request( $path );
	}

	/**
	 * Update metadata file in OC.
	 * @param $uuid
	 * @param $xml
	 * @return mixed
	 */
	private function put_metadata_xml( $uuid, $xml ) {

		$path = 'objects/' . $uuid . '/files/metadata/' . $this->metadata_filename;
		$args = array(
			'method'    => 'PUT',
			'headers'   => array(
				'Content-Type'  => $this->metadata_mimetype
			),
			'body'      => $xml
		);

		return $this->perform_oc_request( $path, $args );
	}

	/**
	 * Performs a HTTP request to Open Content.
	 * @param $path
	 * @param array $http_args
	 * @return null|string
	 */
	private function perform_oc_request( $path, $http_args = array() ) {

		$oc_api         = new OcAPI();
		$req_url        = $oc_api->getOcBaseUrl() . $path;
		$args           = array(
			'headers' => array(
				'Authorization' => 'Basic ' . base64_encode( OC_EDIT_USER . ':' . OC_EDIT_PASSWORD )
			),
			'timeout' => 20
		);

		$args           = array_merge_recursive( $args, $http_args );
		$api_response   = wp_remote_request( $req_url, $args );

		if( !is_wp_error( $api_response ) ) {

			$http_status = wp_remote_retrieve_response_code( $api_response );
			if( $http_status == "200" ) {
				return wp_remote_retrieve_body( $api_response );
			}
			else {
				return null;
			}
		}

		return null;
	}
}